﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Rocket_Defense
{
    /*  Explosion class inherits from SpaceObject.
     *  Contains all values associated with explosion.
     *  This class is used to advance the frames of explosion
     *  sprites.
    */ 
    class Explosion : SpaceObject
    {
        const int _EXPSIZE = 64;        // width and height of explosion sprite
        const int _INTERVAL = 100;      // interval between frames
        const int _TOTALEXPFRAMES = 16; // number of frames in an explosion object
        int _currentFrame;              // current frame of sprite
        int _timeSinceLastFrame;        // time since last frame switch
        Rectangle _expRect;

        public Explosion(Vector2 position)
            :base(position)
        {
            _currentFrame = 0;
            _timeSinceLastFrame = 0;
            _expRect = new Rectangle(_currentFrame * _EXPSIZE, 0, _EXPSIZE, _EXPSIZE);
        }

        /* CurrentFrame: accesser/modifier for _currentFrame. */
        public int CurrentFrame
        {
            get { return _currentFrame; }
            set { _currentFrame = value; }
        }
        /* ExplosionRect: accesser/modifier for _expRect. */
        public Rectangle ExplosionRect
        {
            get {return _expRect;}
            set {_expRect = value;}
        }
        /* atLastFrame: returns true if at last frame, otherwise false */
        public bool atLastFrame()
        {
            return CurrentFrame >= _TOTALEXPFRAMES;
        }

        /* Update: update method for explosion, advances frame when sufficient time has passed. */
        public void Update(GameTime gameTime)
        {
            _timeSinceLastFrame += gameTime.ElapsedGameTime.Milliseconds;
            if (_timeSinceLastFrame > _INTERVAL)
            {
                _timeSinceLastFrame -= _INTERVAL;
                _currentFrame++;
                _expRect.X = _currentFrame*_EXPSIZE;

            }

        }
    }
}
