/*	Eric Tucker
**	CIS 297
**	Final Project
**	Winter 2010
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Rocket_Defense
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D backgroundTexture;        // stores background 
        Texture2D groundTexture;            // stores ground 
        Texture2D startTexture;             // stores start screen 
        Texture2D highscoreTexture;         // stores high screen banner
        Texture2D hsFieldTexture;           // stores high scree field
        Texture2D hsBottomTexture;          // stores bottom message on high score screen
        SpriteFont hsFont;                  // stores font for high scores

        GameManager gameManager;            // class to run game
        const int NUMHIGHSCORES = 10;       // number of high scores stored
        const float HIGHSCORESPACING = 35;  // spacing between high scores on display

        float[] highScores = new float[NUMHIGHSCORES];  // array to hold high scores    
        float lastScore;                    // last score achieved

        Vector2 highScoresPosition = new Vector2(410, 180); // position for first high score 
        Vector2 yourScorePosition = new Vector2(410, 125);  // position for last score to display
        Vector2 hsFieldPosition = new Vector2(400, 115);    // position for high score field
        Vector2 hsBottomPosition = new Vector2(0 , 620);    // position for bottom banner on high score screen
        

        enum GameState { Start, InGame, GameOver };     // 3 possible game states.
        GameState currentGameState;                     // stores current game state
        GamePadState prevPadState;                      // stores previous gamepad state
        GamePadState gamepadState;                      // stores current gamepad state


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            // sets screen size to 720p
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            currentGameState = GameState.Start;
            prevPadState = GamePad.GetState(PlayerIndex.One);
            gameManager = new GameManager(this);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Loads all necessary textures.
            backgroundTexture = Content.Load<Texture2D>(@"images\\background");
            groundTexture = Content.Load<Texture2D>(@"images\\ground");
            startTexture = Content.Load<Texture2D>(@"images\\start");
            highscoreTexture = Content.Load<Texture2D>(@"images\\highscore");
            hsFieldTexture = Content.Load<Texture2D>(@"images\\hsfield");
            hsBottomTexture = Content.Load<Texture2D>(@"images\\hsBottom");
            hsFont = Content.Load<SpriteFont>("hsFont");

            // Populates high scores with zeroes. Was unable to get persistent high scores
            // working on 360, worked properly on PC but had file access issues on 360.
            for (int i = 0; i < highScores.Length; i++)
                    highScores[i] = 0;

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }
       

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            
            // Gets Gamepad state
            gamepadState = GamePad.GetState(PlayerIndex.One);
         
            switch (currentGameState)
            {
                case GameState.Start:   
                    // Starts game when 'A' is pressed.
                    if(gamepadState.Buttons.A == ButtonState.Pressed && prevPadState.Buttons.A == ButtonState.Released)
                    {
                        currentGameState = GameState.InGame;
                        Components.Add(gameManager);
                    }
                    // Switches to High Score Screen when 'B' is pressed.
                    else if (gamepadState.Buttons.B == ButtonState.Pressed && prevPadState.Buttons.B == ButtonState.Released)
                    {
                        gameManager = new GameManager(this);
                        currentGameState = GameState.GameOver;
                    }
                    break;
                case GameState.InGame:
                    // Ends game when all bunkers are destroyed. Advances to High Score Screen
                    if (gameManager.AllBunkersAreDestroyed())
                    {
                        currentGameState = GameState.GameOver;
                        lastScore = gameManager.GetScore();
                        AddToHighScores(lastScore);
                        Components.Remove(gameManager);
                    }
                    // Ends game when all ships are destroyed. Advances to High Score Screen
                    else if (!gameManager.ShipsLeft())
                    {
                        currentGameState = GameState.GameOver;
                        lastScore = gameManager.GetScore();
                        AddToHighScores(lastScore);
                        Components.Remove(gameManager);    
                    }
                    break;
                case GameState.GameOver:
                    // Switches to Start Screen when 'A' is pressed.
                    if (gamepadState.Buttons.A == ButtonState.Pressed && prevPadState.Buttons.A == ButtonState.Released)
                    {
                        gameManager = new GameManager(this);
                        currentGameState = GameState.Start;
                    }
                        break;

                    
            }
            // updates previous gamepad state.
            prevPadState = gamepadState;
            base.Update(gameTime);
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend,
                                SpriteSortMode.FrontToBack, SaveStateMode.None);

            // Draws Background and Ground.
            spriteBatch.Draw(backgroundTexture, Vector2.Zero, null, Color.White,
                0, Vector2.Zero, 1, SpriteEffects.None, 0F);
            spriteBatch.Draw(groundTexture, new Vector2(0,595), null, Color.White,
                            0, Vector2.Zero, 1, SpriteEffects.None, .75F);

            // Draws start screen when GameState = Start
            if(currentGameState.Equals(GameState.Start))
                spriteBatch.Draw(startTexture, Vector2.Zero,null, Color.White,0,Vector2.Zero,1,SpriteEffects.None,1F);

            // Draws High Score Screen when GameState = GameOver
            if (currentGameState.Equals(GameState.GameOver))
            {
       
                string sScore = lastScore.ToString("#,###0");
                spriteBatch.Draw(highscoreTexture, Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, 1F);
                spriteBatch.Draw(hsFieldTexture, hsFieldPosition, null, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, .9F);
                spriteBatch.DrawString(hsFont, "Last score: " + sScore , yourScorePosition, Color.Red, 0, Vector2.Zero, 1F, SpriteEffects.None, 1F);
                spriteBatch.Draw(hsBottomTexture, hsBottomPosition, null, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, .9F);

                for (int i = 0; i < highScores.Length; i++)
                {
                    sScore = highScores[i].ToString("#,###0");
                    spriteBatch.DrawString(hsFont, (i + 1).ToString() + ":" , new Vector2(highScoresPosition.X, highScoresPosition.Y + i * HIGHSCORESPACING),
                                            Color.Red, 0, Vector2.Zero, 1F, SpriteEffects.None, 1F);
                    spriteBatch.DrawString(hsFont, sScore, new Vector2(highScoresPosition.X+50, highScoresPosition.Y + i * HIGHSCORESPACING),
                        Color.Red, 0, Vector2.Zero, 1F, SpriteEffects.None, 1F);
                }




            }
            spriteBatch.End();
            base.Draw(gameTime);
        }

        /* AddToHighScores: adds a float value to highScores array if it is high enough */
        public void AddToHighScores(float score)
        {
            if (score > highScores[NUMHIGHSCORES - 1])
                highScores[NUMHIGHSCORES - 1] = score;

            Array.Sort(highScores);
            Array.Reverse(highScores);

        }

    }
}
