﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
namespace Rocket_Defense
{
    public class GameManager : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /*  GameManager class is one run of the game.
         *  It contains all values and objects to play
         *  Rocket_Defense.
        */

        SpriteBatch spriteBatch;
        Texture2D shipTexture;          // stores ship image
        Texture2D bombTexture;          // stores bomb image
        Texture2D slugTexture;          // stores slug image
        Texture2D missileTexture;       // stores missile image
        Texture2D explosionTexture;     // stores explosion image
        Texture2D cannonTexture;        // stores cannon image
        Texture2D rocketTexture;        // stores rocket image
        Texture2D bunkerTexture;        // stores bunker image
        Texture2D healthTexture;        // stores health bar image
        Texture2D healthBorderTexture;  // stores border image to health bar
        Texture2D pRocketTexture;       // stores plated rocket image
        Texture2D scoreTexture;         // stores score field image
        Texture2D pauseTexture;         // stores pause screen image

        SpriteFont font;                // stores font for score display

        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;
        
        Vector2 base1 = new Vector2(240, 620);      // stores location of first bunker
        Vector2 base2 = new Vector2(640, 620);      // stores location of second bunker
        Vector2 base3 = new Vector2(1040, 620);     // stores location of third bunker
        
        Random rand = new Random();
        List<Ship> shipList = new List<Ship>();         // list of all active ships
        List<Rocket> rocketList = new List<Rocket>();   // list of all active rockets
        List<Bunker> bunkerList = new List<Bunker>();   // list of all bunkers
        List<Vector2> dimensions = new List<Vector2>(); // list of dimensions for ship and ships weapons

        GamePadState prevPadState;      // stores previous gamepad state
        GamePadState gamepadState;      // stores current gamepad state

        const float MAXSHIPS = 100;     // stores maximum number of enemy ships
        const int MAXPLATEDROCKETS = 5; // stores maximum number of plated rockets allowed on screen at once
        const int MAXROCKETS = 15;      // stores maximum number of rockets allowed on screen at once

        float weaponDepth;          // stores depth for weapons textures
        float cannonDepth;          // stores depth for cannon texture
        float shipDepth;            // stores depth for enemy ship texture
        float scoreDepth;           // stores depth for score field texture
        float stringDepth;          // stores depth for strings
        float bunkerDepth;          // stores depth for bunker texture
        float healthBorderDepth;    // stores depth for health border texture
        float healthBarDepth;       // stores depth for health bar
        float expDepth;             // stores depth for explosion texture

        float score;                // stores current score
        float timeSinceLastShip;    // stores time since last ship was added
        float rotateAngle;          // store angle of cannons
        float numShips;             // stores number of ships launched
        float numShipsDestroyed;    // stores number of ships destroyed
        float shipInterval;         // stores interval between ships
        float shipSpeed;            // stores speed of ships

        int numRockets;             // stores number of rockets on screen
        int numPlatedRockets;       // stores number of plated rockets on screen
        int shipDirection;          // stores direction for next ship to go

        bool shipsLeft;             // holds value of whether or not any ships remain
        bool cheat;                 // holds value of whether or not cheat mode is on

        enum GameState { Running, Paused }; // store 2 possible game states
        GameState currentGameState;         // current game state

        public GameManager(Game game)
            : base(game)
        {
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // Set Initial Values.
            currentGameState = GameState.Running;
            prevPadState = GamePad.GetState(PlayerIndex.One);
            score = 0;
            timeSinceLastShip = 0;
            rotateAngle = 0;
            numShips = 2;
            shipDirection = 1;
            shipSpeed = 1;
            numShipsDestroyed = 0;
            cheat = false;
            numRockets = 0;
            numPlatedRockets = 0;
            shipInterval = 20000;
            shipsLeft = true;

            weaponDepth = .925F;
            cannonDepth = .9F;
            shipDepth = .99F;
            scoreDepth = .9F;
            stringDepth = 1F;
            bunkerDepth = .8F;
            healthBorderDepth = 9F;
            healthBarDepth = .99F;
            expDepth = 1F;

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(Game.GraphicsDevice);

            // Load Textures.
            shipTexture = Game.Content.Load<Texture2D>(@"images\\smallShip");
            bombTexture = Game.Content.Load<Texture2D>(@"images\\bomb");
            slugTexture = Game.Content.Load<Texture2D>(@"images\\slug");
            missileTexture = Game.Content.Load<Texture2D>(@"images\\missile");
            explosionTexture = Game.Content.Load<Texture2D>("explosion");
            cannonTexture = Game.Content.Load<Texture2D>(@"images\\cannon");
            rocketTexture = Game.Content.Load<Texture2D>(@"images\\rocket");
            pRocketTexture = Game.Content.Load<Texture2D>(@"images\\platedRocket");
            bunkerTexture = Game.Content.Load<Texture2D>(@"images\\bunker");
            healthBorderTexture = Game.Content.Load<Texture2D>(@"images\\hpborder");
            healthTexture = Game.Content.Load<Texture2D>(@"images\\hp");
            scoreTexture = Game.Content.Load<Texture2D>(@"images\\score");
            pauseTexture = Game.Content.Load<Texture2D>(@"images\\pause");

            // Load Audio
            audioEngine = new AudioEngine(@"Content\audio\gameSounds.xgs");
            waveBank = new WaveBank(audioEngine, @"Content\audio\Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, @"Content\audio\Sound Bank.xsb");

            font = Game.Content.Load<SpriteFont>("myFont");

            // Store object dimensions for ships.
            dimensions.Add(new Vector2(shipTexture.Width, shipTexture.Height));
            dimensions.Add(new Vector2(bombTexture.Width, bombTexture.Height));
            dimensions.Add(new Vector2(slugTexture.Width, slugTexture.Height));
            dimensions.Add(new Vector2(missileTexture.Width, missileTexture.Height));

            // create bunkers
            bunkerList.Add(new Bunker(base1, bunkerTexture.Width, bunkerTexture.Height));
            bunkerList.Add(new Bunker(base2, bunkerTexture.Width, bunkerTexture.Height));
            bunkerList.Add(new Bunker(base3, bunkerTexture.Width, bunkerTexture.Height));

            // create initial 2 ships.
            shipList.Add(new Ship(GetShipPos(), GetShipDirection(), GetShipDepth(), shipSpeed, dimensions));
            shipList.Add(new Ship(GetShipPos(), GetShipDirection(), GetShipDepth(), shipSpeed, dimensions));


            base.LoadContent();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {

            gamepadState = GamePad.GetState(PlayerIndex.One);

            switch (currentGameState)
            {
                case GameState.Running:
                {
                    AddShips(gameTime.ElapsedGameTime.Milliseconds);

                    // update rockets.
                    for (int i = 0; i < rocketList.Count(); i++)
                    {
                        rocketList[i].Update();
                        if (rocketList[i].IsOffScreen())
                           RemoveRocket(i);   
                    }

                    // update ships and their weapons
                    for (int i = 0; i < shipList.Count(); i++)
                    {
                        shipList[i].Update( gameTime, bunkerList);

                        DetectCollisions(i);

                        DetectShipCollision(i);
                    }

                    /* <CONTROLLER INPUT> */
                    if(gamepadState.DPad.Left == ButtonState.Pressed)
                        rotateAngle -= .05f;
                    if (gamepadState.DPad.Right == ButtonState.Pressed)
                        rotateAngle += .05f;
                    rotateAngle += gamepadState.ThumbSticks.Left.X * .05f;

                    if (gamepadState.Buttons.X == ButtonState.Pressed && ((prevPadState.Buttons.X == ButtonState.Released 
                        && numRockets < MAXROCKETS && bunkerList[0].Exists()) || cheat))
                        LaunchRocket(base1, false);

                    if (gamepadState.Buttons.A == ButtonState.Pressed && ((prevPadState.Buttons.A == ButtonState.Released 
                        && numRockets < MAXROCKETS && bunkerList[1].Exists()) || cheat))
                        LaunchRocket(base2, false);

                    if (gamepadState.Buttons.B == ButtonState.Pressed && ((prevPadState.Buttons.B == ButtonState.Released 
                        && numRockets < MAXROCKETS && bunkerList[2].Exists()) || cheat))
                        LaunchRocket(base3, false);

                    if (gamepadState.Buttons.Y == ButtonState.Pressed && ((prevPadState.Buttons.Y == ButtonState.Released 
                        && numPlatedRockets < MAXPLATEDROCKETS && bunkerList[1].Exists()) || cheat))
                        LaunchRocket(base2, true);

                    if(gamepadState.Buttons.RightStick == ButtonState.Pressed && prevPadState.Buttons.RightStick == ButtonState.Released)
                        cheat = !cheat;
                    /* </CONTROLLER INPUT> */

                    // Limits cannon to 180 degrees of movement
                    if (rotateAngle > MathHelper.PiOver2)
                        rotateAngle = MathHelper.PiOver2;
                    if (rotateAngle < -MathHelper.PiOver2)
                        rotateAngle = -MathHelper.PiOver2;
                    
                }
                break;

            }

            // Pauses game when running, unpauses when paused.
            if(gamepadState.Buttons.Start == ButtonState.Pressed && prevPadState.Buttons.Start == ButtonState.Released)
            {
                if (currentGameState.Equals(GameState.Running))
                    currentGameState = GameState.Paused;
                else
                    currentGameState = GameState.Running;
            }
           
            prevPadState = gamepadState;

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend,
                    SpriteSortMode.FrontToBack, SaveStateMode.None);

            // Draw score field and current score.
            spriteBatch.Draw(scoreTexture, new Vector2(0, 670), null, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, scoreDepth);
            spriteBatch.DrawString(font, score.ToString(), new Vector2(5, 695), Color.Red, 0, Vector2.Zero, 1F, SpriteEffects.None, stringDepth);

            // Draws Bunkers, attached cannons and health bar display
            for (int i = 0; i < bunkerList.Count(); i++)
            {
                if (bunkerList[i].Exists())
                {

                   spriteBatch.Draw(bunkerTexture, bunkerList[i].DrawPos, null, Color.White, 0,
                                    new Vector2(bunkerTexture.Width / 2, bunkerTexture.Height / 4), 1, SpriteEffects.None, bunkerDepth);
                    spriteBatch.Draw(cannonTexture, bunkerList[i].DrawPos, null, Color.White, rotateAngle,
                                    new Vector2(cannonTexture.Width / 2, cannonTexture.Height), 1, SpriteEffects.None, cannonDepth);
                    spriteBatch.Draw(healthBorderTexture, new Vector2(bunkerList[i].Position.X, bunkerList[i].Position.Y + 100), null, Color.White, 0,
                                    Vector2.Zero, 1, SpriteEffects.None, healthBorderDepth);
                    spriteBatch.Draw(healthTexture, new Vector2(bunkerList[i].Position.X, bunkerList[i].Position.Y + 100),
                                    new Rectangle(0, 0, (int)(healthTexture.Width * bunkerList[i].GetHealthPercent()), healthTexture.Height), 
                                    Color.White, 0, Vector2.Zero, 1,SpriteEffects.None, healthBarDepth);
                                    
                }
            }

            // Draw Ships & Weapons
            for (int i = 0; i < shipList.Count(); i++)
            {
                spriteBatch.Draw(shipTexture, shipList[i].Position, null, Color.White,
                                    0, Vector2.Zero, 1, SpriteEffects.None, shipList[i].Depth);

                for (int j = 0; j < shipList[i].GetBombList().Count; j++)
                    spriteBatch.Draw(bombTexture, shipList[i].GetBombList()[j].Position, null, Color.White,
                                       0, Vector2.Zero, 1, SpriteEffects.None, weaponDepth);

                for (int j = 0; j < shipList[i].GetSlugList().Count; j++)
                    spriteBatch.Draw(slugTexture, shipList[i].GetSlugList()[j].Position, null, Color.White,
                                       0, Vector2.Zero, 1, SpriteEffects.None, weaponDepth);

                for (int j = 0; j < shipList[i].GetMissileList().Count; j++)
                    spriteBatch.Draw(missileTexture, shipList[i].GetMissileList()[j].Position, null, Color.White,
                                       shipList[i].GetMissileList()[j].Angle, new Vector2(missileTexture.Width / 2, missileTexture.Height / 2), 1, SpriteEffects.None, weaponDepth);

                for (int j = 0; j < shipList[i].GetExpList().Count; j++)
                    spriteBatch.Draw(explosionTexture, shipList[i].GetExpList()[j].Position, shipList[i].GetExpList()[j].ExplosionRect, Color.White,
                            0, Vector2.Zero, 1, SpriteEffects.None, expDepth);

            }

            // Draw rockets
            for (int i = 0; i < rocketList.Count; i++)
                if(!rocketList[i].Plated)
                    spriteBatch.Draw(rocketTexture, rocketList[i].Position, null, Color.White, rocketList[i].Angle,
                            new Vector2(rocketTexture.Width / 2, rocketTexture.Height), 1, SpriteEffects.None, cannonDepth);
                else
                    spriteBatch.Draw(pRocketTexture, rocketList[i].Position, null, Color.White, rocketList[i].Angle,
                            new Vector2(rocketTexture.Width / 2, rocketTexture.Height), 1, SpriteEffects.None, cannonDepth);

            // Draws pause screen if paused.
            if(currentGameState.Equals(GameState.Paused))
                spriteBatch.Draw(pauseTexture, Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, 1);

            spriteBatch.End();
            base.Draw(gameTime);
        }


        /* Detect Collisions: accepts an integer value i, containing the index of a
         * ship in List of Ships. DetectCollisions tests all rockets currently active
         * to see if they collide with any of the weapons dropped by ship at index i
        */
        public void DetectCollisions(int i)
        {
            for (int j = 0; j < rocketList.Count(); j++)
            {
                bool rocketExists = true;
                // Checks for collisions with each bomb
                for (int k = 0; k < shipList[i].GetBombList().Count() && rocketExists; k++)
                {
                    if (rocketList[j].Collides(shipList[i].GetBombList()[k]))
                    {
                        shipList[i].GetExpList().Add(new Explosion(new Vector2(shipList[i].GetBombList()[k].Position.X - bombTexture.Width, shipList[i].GetBombList()[k].Position.Y)));
                        shipList[i].GetExpList().Add(new Explosion(new Vector2(rocketList[j].Position.X - rocketTexture.Width, rocketList[j].Position.Y - rocketTexture.Height)));
                        score += shipList[i].GetBombList()[k].Points;
                        shipList[i].GetBombList().RemoveAt(k);
                        RemoveRocket(j);

                        //soundBank.PlayCue("shortExplode");

                        rocketExists = false;
                    }
                }
                // checks for collisions with each slug
                for (int k = 0; k < shipList[i].GetSlugList().Count() && rocketExists; k++)
                {
                    if (rocketList[j].Collides(shipList[i].GetSlugList()[k]))
                    {
                        shipList[i].GetExpList().Add(new Explosion(new Vector2(shipList[i].GetSlugList()[k].Position.X - slugTexture.Width / 2, shipList[i].GetSlugList()[k].Position.Y)));
                        shipList[i].GetExpList().Add(new Explosion(new Vector2(rocketList[j].Position.X - rocketTexture.Width, rocketList[j].Position.Y - rocketTexture.Height)));
                        score += shipList[i].GetSlugList()[k].Points;
                        shipList[i].GetSlugList().RemoveAt(k);
                        RemoveRocket(j);
                        //soundBank.PlayCue("shortExplode");

                        rocketExists = false;
                    }
                }
                // checks for collisions with each missile
                for (int k = 0; k < shipList[i].GetMissileList().Count() && rocketExists; k++)
                {
                    if (rocketList[j].Collides(shipList[i].GetMissileList()[k]))
                    {
                        shipList[i].GetExpList().Add(new Explosion(new Vector2(shipList[i].GetMissileList()[k].Position.X - missileTexture.Width * 2, shipList[i].GetMissileList()[k].Position.Y)));
                        shipList[i].GetExpList().Add(new Explosion(new Vector2(rocketList[j].Position.X - rocketTexture.Width, rocketList[j].Position.Y - rocketTexture.Height)));
                        score += shipList[i].GetMissileList()[k].Points;
                        shipList[i].GetMissileList().RemoveAt(k);
                        RemoveRocket(j);
                        //soundBank.PlayCue("shortExplode");

                        rocketExists = false;
                    }
                }


            }
        }


        /* DetectShipCollisions: accepts an integer value i, containing the index of a
         * ship in List of Ships. DetectShipCollisions tests all rockets currently active
         * to see if they collide with an active ship. If they are plated, they damage the
         * ship. If they take total health below 0, it destroys the ship.
        */
        public void DetectShipCollision(int i)
        {
            for (int j = 0; j < rocketList.Count(); j++)
            {
                // continues only if rocket is plated.
                if (rocketList[j].Collides(shipList[i]) && rocketList[j].Plated)
                {
                    // damages ship
                    shipList[i].Health -= rocketList[j].Damage;
                    shipList[i].GetExpList().Add(new Explosion(new Vector2(rocketList[j].Position.X - rocketTexture.Width, rocketList[j].Position.Y - rocketTexture.Height * 2)));
                    //soundBank.PlayCue("shortExplode");
                    RemoveRocket(j);

                    // destroys ship if below 0 health.
                    if (shipList[i].Health <= 0)
                    {
                        numShipsDestroyed++;
                        shipInterval -= 150;                                // decreases interval between ships
                        shipSpeed *= (numShipsDestroyed / MAXSHIPS/18 + 1); // speeds up next enemy
                        score += shipList[i].Points;
                        ExplodeShip(shipList[i].Position, i);                        
                        soundBank.PlayCue("shipExplode");
                        // if more ships are available, transfers control of bombList,slugList,missileList and expList to next new ship
                        if (numShips < MAXSHIPS)
                        {
                            numShips++;
                            shipList.Add(new Ship(GetShipPos(), GetShipDirection(), GetShipDepth(), shipSpeed, shipList[i].GetBombList(), shipList[i].GetSlugList(),
                                            shipList[i].GetMissileList(), shipList[i].GetExpList(), dimensions));
                        }
                        // else if no new ships are available, transfers bombList,slugList,missileList and expList to existing ship
                        else if (shipList.Count >= 2)
                        {
                            if(i != 0)
                                shipList[0].AddLists(shipList[i].GetBombList(), shipList[i].GetSlugList(), shipList[i].GetMissileList(), shipList[i].GetExpList());
                            else
                                shipList[1].AddLists(shipList[i].GetBombList(), shipList[i].GetSlugList(), shipList[i].GetMissileList(), shipList[i].GetExpList());

                        }
                        else
                        {
                            shipsLeft = false;
                        }
                        shipList.RemoveAt(i);
                        break;
                    }

                }
            }
        }
        /* ShipsLeft: accessor for bool shipsLeft */
        public bool ShipsLeft() { return shipsLeft; }

        /* GetScore: accessor for float score */
        public float GetScore() { return score; }

        /* GetShipDepth: returns a float value containing always decreasing depth value for new ship */
        public float GetShipDepth()
        {   return (shipDepth -= .0001F); }

        /* GetShipPos: returns a Vector2 value containing slightly random starting position for new ship */
        public Vector2 GetShipPos()
        {   return new Vector2(rand.Next(1130) - 125, 0+ rand.Next(50));  }

        /* GetShipDirection: returns a int value that alternates between positive and negative for ship initial direction */
        public int GetShipDirection()
        {    return (shipDirection *= -1);  }

        /* AllBunkersAreDestroyed: returns a bool value containing true if all bunkers are destroyed, false otherwise. */
        public bool AllBunkersAreDestroyed()
        {
            bool dead = true;
            for (int i = 0; i < bunkerList.Count(); i++)
                if(bunkerList[i].Exists())
                    dead = false;
            
            return dead;
            
        }

        /* ExplodeShip: accepts a Vector2 ship position and an int value containing a ship index.
         * It create multiple explosions over the space occupied by the exploding ship.
        */
        public void ExplodeShip(Vector2 pos, int ship)
        {
            int offset;
            for(int i = 0; i < shipTexture.Width; i+=shipTexture.Width/10)
                for (int j = -50; j < shipTexture.Height-50; j += shipTexture.Height/10)
                {
                    offset = rand.Next(50);
                    shipList[ship].GetExpList().Add(new Explosion(new Vector2(pos.X + i + offset / 2, pos.Y + j + offset)));
                }
        }

        /* RemoveRocket: Removes rocket at int index in RocketList */
        public void RemoveRocket(int index)
        {
            if (rocketList[index].Plated)
                numPlatedRockets--;
            else
                numRockets--;

            rocketList.RemoveAt(index);
            soundBank.PlayCue("shortExplode");
        }

        /* AddShips: accepts float input of time, creates new ship if enough time has passed and ships are available */
        public void AddShips(float time)
        {
            timeSinceLastShip += time;
            if (timeSinceLastShip > shipInterval && numShips < MAXSHIPS)
            {
                shipList.Add(new Ship(GetShipPos(),GetShipDirection(),GetShipDepth(), shipSpeed, dimensions));
                numShips++;
                timeSinceLastShip -= shipInterval;
            }
        }

        /* LaunchRocket: accept Vector2 of bunker location and bool value plated. Adds new rocket according to 
         * position and plated value.
        */
        public void LaunchRocket(Vector2 basePosition, bool plated)
        {
            if (!plated)
            {
                rocketList.Add(new Rocket(basePosition, rotateAngle, rocketTexture.Width, rocketTexture.Height));
                numRockets++;
            }
            else
            {
                rocketList.Add(new Rocket(basePosition, rotateAngle, rocketTexture.Width, rocketTexture.Height, true));
                numPlatedRockets++;
            }
            
            soundBank.PlayCue("smallExplode");
        }

    }

}
