﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Rocket_Defense
{
    /*  Rocket class inherits from SpaceWeapon.
     *  It contains all information associated with
     *  rockets launched from defensive bunkers. This
     *  class is used to store all necessary information
     *  to advance rockets at the proper trajectory.
    */ 
    class Rocket : SpaceWeapon
    {
        const float _LAUNCHDIST = 40;   // distance from center of bunker to launch rocket.
        const float _PLATEDSPEED = 4;   // speed of plate rocket.
        const float _ROCKETSPEED = 2;   // speed of standard rocket.
        protected float _angle;         // angle rocket was launched at
        Vector2 _direction;             // calculated direction for rockets movement
        bool _plated;                   // stores whether or not rocket is plated.

        /* Default constructor */
        public Rocket(Vector2 position, float angle, float width, float height)
            :base(position,width,height)
        {
            _angle = angle;
            _direction = new Vector2((float)Math.Sin(angle), -(float)Math.Cos(angle));
            _position += _LAUNCHDIST * _direction;
            _damage = 100;
            _speed = _ROCKETSPEED;
            _plated = false;
        }

        /* Constructor used for plated rockets */
        public Rocket(Vector2 pos, float angle, float width, float height, bool plated)
            : this (pos,angle,width,height)
        {
            _plated = plated;
            _speed = _PLATEDSPEED;
        }

        /* Angle: accessor/modifier for _angle. */
        public float Angle
        {
            get { return _angle; }
            set { _angle = value; }
        }

        /* Plated: accessor/modifier for _plated. */
        public bool Plated
        {
            get { return _plated; }
            set { _plated = value; }
        }

        /* Update: updated position of rocket. */
        public override void Update()
        {
           _position += _direction * _speed;

        }

        /* IsOffScreen: returns true if rocket has moved off screen, false otherwise */
        public bool IsOffScreen()
        {
            return (Position.X < 0 || Position.Y < 0 || Position.X > 1280);
        }
    }
}
