﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;


namespace Rocket_Defense
{
    class Ship : SpaceWeapon
    {
        const int _GROUND = 620;        // y value for ground location
        const float _MAXHEALTH = 1000;  // stores max health for a ship
        const int _MAXBOMBS = 25;       // stores max bombs ship can launch
        const int _MAXSLUGS = 10;       // stores max slugs ship can launch
        const int _MAXMISSILES = 5;     // stores max missiles ship can launch
        const int _COUNTERINTERVAL = 5; // interval used to help offset launches
        const float _BDELAY = 50;       // stores delay between bombs
        const float _SDELAY = 100;      // stores delay between slugs
        const float _MDELAY = 125;      // stores delay between missiles

        static float _yOffset = 100;    // stores y offset for ships starting location.
        Random rand = new Random();

        int _direction;                 // stores ships direction
        int _counter;                   // stores counter for weapon delays
        float _bombDelay;               // stores changing bomb delay
        float _slugDelay;               // stores changing slug delay
        float _missileDelay;            // stores changing missile delay
        float _bombWidth;               // stores bomb width
        float _bombHeight;              // stores bomb height
        float _slugWidth;               // stores slug width
        float _missileWidth;            // stores missile width
        float _missileHeight;           // stores missile height
        float _shipYOffset;             // stores current Y offset of ship

        List<Bomb> _bombList = new List<Bomb>();            // stores list of all active bombs
        List<Slug> _slugList = new List<Slug>();            // stores list of all active slugs
        List<Missile> _missileList = new List<Missile>();   // stores list of all active missiles
        List<Explosion> _expList = new List<Explosion>();   // stores list of all active explosions
        List<Bunker> _bunkerList = new List<Bunker>();      // stores list of all bunkers

        /* <CONSTUCTORS> */
        public Ship(Vector2 pos, int direction, float depth, float speed, List<Vector2> dimensions)
            :base(new Vector2(pos.X, pos.Y-_yOffset), dimensions[0].X,dimensions[0].Y, speed, depth)
        {

            _direction = direction;
            _points = 1000;
            _health = _MAXHEALTH;
            _bombDelay = _BDELAY;
            _slugDelay = _SDELAY;
            _missileDelay = _MDELAY;
            _counter = 5;
            _shipYOffset = _yOffset;
            _bombWidth = dimensions[1].X;
            _bombHeight = dimensions[1].Y;
            _slugWidth = dimensions[2].X;
            _missileWidth = dimensions[3].X;
            _missileHeight = dimensions[3].Y;

        }

        public Ship(Vector2 pos, int direction, float depth, float speed, List<Bomb> bombList,
                    List<Slug> slugList, List<Missile> missileList, List<Explosion> expList, List<Vector2> dimensions)
            : this(pos, direction, depth, speed, dimensions)
        {
            _bombList = bombList;
            _slugList = slugList;
            _missileList = missileList;
            _expList = expList;
        }
        /* </CONSTUCTORS> */

        /* GetBombList: accessor for _bombList */
        public List<Bomb> GetBombList() { return _bombList;}
        /* GetSlugList: accessor for _slugList */
        public List<Slug> GetSlugList() { return _slugList;}
        /* GetMissileList: accessor for _missileList */
        public List<Missile> GetMissileList() { return _missileList;}
        /* GetExpList: accessor for _expList */
        public List<Explosion> GetExpList() { return _expList;}

        /* Direction: accessor/modifier for _direction */
        public int Direction
        {
            get { return _direction; }
            set { _direction = value; }
        }

        /* Update: Updates location of ship and all children objects. Launchs
         * new weapons if sufficient time has passed.
        */
        public void Update(GameTime gameTime, List<Bunker> bunkerList)
        {
            _bunkerList = bunkerList;

            UpdateShipLocation();

            // update bomb positions
            UpdateWeaponPositions();

            // update explosions
            UpdateExplosions(gameTime);

            // launch weapons
            LaunchWeapons();
         
        }
        /* UpdateShipLocation: changes ships location according to ships current speed and direction */
        public void UpdateShipLocation()
        {
            if (_position.X <= (0 - (_width * .33F)))
                _direction = 1;
            if (_position.X >= 1280 - _width * .66F)
                _direction = -1;
            if (_shipYOffset >= 0)
            {
                _position.Y++;
                _shipYOffset--;
            }

            _position.X += _direction * _speed;
        }

        /* UpdateExplosions: updates explosions and removes them when they reach the last sprite frame. */
        public void UpdateExplosions(GameTime gameTime)
        {
            for (int i = 0; i < _expList.Count; i++)
            {
                _expList[i].Update(gameTime);
                if (_expList[i].atLastFrame())
                    _expList.RemoveAt(i);
            }
        }

        /* LaunchWeapons: uses static(_counter) and random delay to stagger and delay weapons launch */
        public void LaunchWeapons()
        {
            if (_counter < 0)
            {
                LaunchBombs(rand.Next(5));
                LaunchSlugs(rand.Next(5));
                LaunchMissiles(rand.Next(5));
                _counter = _COUNTERINTERVAL;
            }

            _counter--;
        }

        /* LaunchMissiles: accepts an int delay, launches new missile if set interval(_missileDelay) has passed. */
        public void LaunchMissiles(int delay)
        {
            _missileDelay -= delay;

            if (_missileList.Count < _MAXMISSILES && _missileDelay < 0)
            {
                _missileList.Add(new Missile(new Vector2(Position.X + _width * .47F, Position.Y + _height), TargetBase(), _missileWidth, _missileHeight, _speed));
                _missileDelay = _MDELAY;
            }
        }

        /* LaunchSlug: accepts an int delay, launches new slug if set interval(_slugDelay) has passed. */
        public void LaunchSlugs(int delay)
        {
            _slugDelay -= delay;

            if (_slugList.Count < _MAXSLUGS && _slugDelay < 0)
            {
                _slugList.Add(new Slug(new Vector2(Position.X + _width*.47F, Position.Y + _height), _slugWidth, _slugWidth, _speed));
                _slugDelay = _SDELAY;
            }
        }

        /* LaunchBombs: accepts an int delay, launches new bomb if set interval(_bombDelay) has passed. */
        public void LaunchBombs(int delay)
        {
            _bombDelay -= delay;

            if (_bombList.Count < _MAXBOMBS && _bombDelay < 0)
            {
                _bombList.Add(new Bomb(new Vector2(Position.X + _width * .47F, Position.Y + .66F*_height), _bombWidth, _bombWidth, Speed));
                _bombDelay = _BDELAY;
            }
        }

        /* TargetBase: returns a vector of a bunker location for targetting, vector value depends on current
         * location of ship and which bunkers exist
        */
        public Vector2 TargetBase()
        {
            if (Position.X < 300)
            {
                if (_bunkerList[0].Exists())
                    return _bunkerList[0].DrawPos;
                else if (_bunkerList[1].Exists())
                    return _bunkerList[1].DrawPos;
                else
                    return _bunkerList[2].DrawPos;
            }
            else if (Position.X < 625)
            {
                if (_bunkerList[1].Exists())
                    return _bunkerList[1].DrawPos;
                else if (_bunkerList[2].Exists())
                    return _bunkerList[2].DrawPos;
                else
                    return _bunkerList[0].DrawPos;
            }
            else
            {
                if (_bunkerList[2].Exists())
                    return _bunkerList[2].DrawPos;
                else if (_bunkerList[1].Exists())
                    return _bunkerList[1].DrawPos;
                else
                    return _bunkerList[0].DrawPos;
            }

        }
        /* CollidesWithBunker: returns true if sw colldies with a bunker, false otherwise.
         * Also, damages bunker accordingly.
        */
        public bool CollidesWithBunker(SpaceWeapon sw)
        {
            for (int i = 0; i < _bunkerList.Count(); i++)
            {
                if (sw.Collides(_bunkerList[i]) && _bunkerList[i].Exists())
                {
                    _bunkerList[i].Health -= sw.Damage;
                    return true;
                }
            }
            return false;

        }

        /* AddLists: accept List<Bomb>, List<Slug>, List<Missile>, List<Explosion>, with primary purpose
         * of transferring weapons lists from destroyed ship to existing ship so they are not removed from
         * the screen.
        */
        public void AddLists(List<Bomb> bombList, List<Slug> slugList, List<Missile> missileList, List<Explosion> expList)
        {
            _bombList.AddRange(bombList);
            _slugList.AddRange(slugList);
            _missileList.AddRange(missileList);
            _expList.AddRange(expList);
        }

        /* UpdateWeaponPositions: updates positions of all weapons tracked by ship. */
        public void UpdateWeaponPositions()
        {
            // updates bomb positions
            for (int i = 0; i < _bombList.Count; i++)
            {
                _bombList[i].Update();
                if (CollidesWithBunker(_bombList[i]))
                {
                    _expList.Add(new Explosion(new Vector2(_bombList[i].Position.X - _bombWidth, _bombList[i].Position.Y)));
                    _bombList.RemoveAt(i);
                }
                else if (_bombList[i].Position.Y >= _GROUND)
                {
                    _expList.Add(new Explosion(new Vector2(_bombList[i].Position.X - _bombWidth, _bombList[i].Position.Y)));
                    _bombList.RemoveAt(i);

                }
            }
            // update slug positions
            for (int i = 0; i < _slugList.Count; i++)
            {
                _slugList[i].Update();
                if (CollidesWithBunker(_slugList[i]))
                {
                    _expList.Add(new Explosion(new Vector2(_slugList[i].Position.X - _slugWidth / 2, _slugList[i].Position.Y)));
                    _slugList.RemoveAt(i);
                }
                else if (_slugList[i].Position.Y >= _GROUND)
                {
                    _expList.Add(new Explosion(new Vector2(_slugList[i].Position.X - _slugWidth / 2, _slugList[i].Position.Y)));
                    _slugList.RemoveAt(i);
                }
            }

            // update missile positions
            for (int i = 0; i < _missileList.Count; i++)
            {
                _missileList[i].Update();

                if (CollidesWithBunker(_missileList[i]))
                {
                    _expList.Add(new Explosion(new Vector2(_missileList[i].Position.X - _missileWidth * 2, _missileList[i].Position.Y)));
                    _missileList.RemoveAt(i);
                }
                else if (_missileList[i].Position.Y >= _GROUND)
                {
                    _expList.Add(new Explosion(new Vector2(_missileList[i].Position.X - _missileWidth * 2, _missileList[i].Position.Y)));
                    _missileList.RemoveAt(i);
                }
            }
        }
    }
}
